# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
import typing_extensions
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BlockUpdate(UncheckedBaseModel):
    """
    Update a block
    """

    value: typing.Optional[str] = pydantic.Field(default=None)
    """
    Value of the block.
    """

    limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    Character limit of the block.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the block if it is a template.
    """

    is_template: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the block is a template (e.g. saved human/persona options).
    """

    label: typing.Optional[str] = pydantic.Field(default=None)
    """
    Label of the block (e.g. 'human', 'persona') in the context window.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the block.
    """

    metadata: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="metadata_")
    ] = pydantic.Field(default=None)
    """
    Metadata of the block.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ActionModel,
    ActionParametersModel,
    ActionResponseModel,
    AgentEnvironmentVariable,
    AgentState,
    AgentStateToolRulesItem,
    AgentType,
    AppAuthScheme,
    AppAuthSchemeAuthMode,
    AppModel,
    ArchivalMemorySummary,
    AssistantFile,
    AssistantMessageInput,
    AssistantMessageOutput,
    AuthRequest,
    AuthResponse,
    AuthSchemeField,
    Block,
    BlockUpdate,
    ChatCompletionRequest,
    ChatCompletionRequestFunctionCall,
    ChatCompletionRequestMessagesItem,
    ChatCompletionRequestStop,
    ChatCompletionRequestToolChoice,
    ChatCompletionResponse,
    ChildToolRule,
    Choice,
    ConditionalToolRule,
    ConflictErrorBody,
    ContextWindowOverview,
    CreateAssistantFileRequest,
    CreateAssistantRequest,
    CreateBlock,
    DeleteAssistantFileResponse,
    DeleteAssistantResponse,
    E2BSandboxConfig,
    EmbeddingConfig,
    EmbeddingConfigEmbeddingEndpointType,
    FileMetadata,
    FunctionCallInput,
    FunctionCallOutput,
    FunctionSchema,
    Health,
    HttpValidationError,
    InitToolRule,
    InternalServerErrorBody,
    Job,
    JobStatus,
    LettaRequest,
    LettaResponse,
    LettaResponseMessagesItem,
    LettaResponseMessagesItem_AssistantMessage,
    LettaResponseMessagesItem_ReasoningMessage,
    LettaResponseMessagesItem_SystemMessage,
    LettaResponseMessagesItem_ToolCallMessage,
    LettaResponseMessagesItem_ToolReturnMessage,
    LettaResponseMessagesItem_UserMessage,
    LettaSchemasLettaMessageToolCall,
    LettaSchemasMessageMessage,
    LettaSchemasOpenaiChatCompletionRequestTool,
    LettaSchemasOpenaiChatCompletionRequestToolCall,
    LettaSchemasOpenaiChatCompletionRequestToolCallFunction,
    LettaSchemasOpenaiChatCompletionResponseMessage,
    LettaSchemasOpenaiChatCompletionResponseToolCall,
    LettaSchemasOpenaiChatCompletionsToolCallFunction,
    LettaSchemasOpenaiChatCompletionsToolCallInput,
    LettaSchemasOpenaiChatCompletionsToolCallOutput,
    LettaSchemasToolTool,
    LettaUsageStatistics,
    LlmConfig,
    LlmConfigModelEndpointType,
    LocalSandboxConfig,
    LogProbToken,
    Memory,
    MessageContentLogProb,
    MessageCreate,
    MessageCreateRole,
    MessageRole,
    NotFoundErrorBody,
    NotFoundErrorBodyMessage,
    OpenAiAssistant,
    Organization,
    OrganizationCreate,
    Passage,
    Provider,
    ReasoningMessage,
    RecallMemorySummary,
    ResponseFormat,
    SandboxConfig,
    SandboxConfigCreate,
    SandboxConfigCreateConfig,
    SandboxConfigUpdate,
    SandboxConfigUpdateConfig,
    SandboxEnvironmentVariable,
    SandboxEnvironmentVariableCreate,
    SandboxEnvironmentVariableUpdate,
    SandboxType,
    Source,
    SystemMessageInput,
    SystemMessageOutput,
    TerminalToolRule,
    ToolCallDelta,
    ToolCallFunctionOutput,
    ToolCallMessage,
    ToolCallMessageToolCall,
    ToolCreate,
    ToolFunctionChoice,
    ToolInput,
    ToolMessage,
    ToolReturnMessage,
    ToolReturnMessageStatus,
    ToolRuleType,
    ToolType,
    UsageStatistics,
    User,
    UserCreate,
    UserMessageInput,
    UserMessageInputContent,
    UserMessageOutput,
    UserUpdate,
    ValidationError,
    ValidationErrorLocItem,
)
from .errors import ConflictError, InternalServerError, NotFoundError, UnprocessableEntityError
from . import agents, blocks, health, jobs, models, providers, sources, tools
from .agents import (
    AgentsSearchRequestCombinator,
    AgentsSearchRequestSearchItem,
    AgentsSearchRequestSearchItemName,
    AgentsSearchRequestSearchItemNameOperator,
    AgentsSearchRequestSearchItemOrderBy,
    AgentsSearchRequestSearchItemOrderByDirection,
    AgentsSearchRequestSearchItemOrderByValue,
    AgentsSearchRequestSearchItemVersion,
    AgentsSearchRequestSearchItem_Name,
    AgentsSearchRequestSearchItem_OrderBy,
    AgentsSearchRequestSearchItem_Version,
    CreateAgentRequestToolRulesItem,
    UpdateAgentToolRulesItem,
)
from .client import AsyncLetta, Letta
from .environment import LettaEnvironment
from .version import __version__

__all__ = [
    "ActionModel",
    "ActionParametersModel",
    "ActionResponseModel",
    "AgentEnvironmentVariable",
    "AgentState",
    "AgentStateToolRulesItem",
    "AgentType",
    "AgentsSearchRequestCombinator",
    "AgentsSearchRequestSearchItem",
    "AgentsSearchRequestSearchItemName",
    "AgentsSearchRequestSearchItemNameOperator",
    "AgentsSearchRequestSearchItemOrderBy",
    "AgentsSearchRequestSearchItemOrderByDirection",
    "AgentsSearchRequestSearchItemOrderByValue",
    "AgentsSearchRequestSearchItemVersion",
    "AgentsSearchRequestSearchItem_Name",
    "AgentsSearchRequestSearchItem_OrderBy",
    "AgentsSearchRequestSearchItem_Version",
    "AppAuthScheme",
    "AppAuthSchemeAuthMode",
    "AppModel",
    "ArchivalMemorySummary",
    "AssistantFile",
    "AssistantMessageInput",
    "AssistantMessageOutput",
    "AsyncLetta",
    "AuthRequest",
    "AuthResponse",
    "AuthSchemeField",
    "Block",
    "BlockUpdate",
    "ChatCompletionRequest",
    "ChatCompletionRequestFunctionCall",
    "ChatCompletionRequestMessagesItem",
    "ChatCompletionRequestStop",
    "ChatCompletionRequestToolChoice",
    "ChatCompletionResponse",
    "ChildToolRule",
    "Choice",
    "ConditionalToolRule",
    "ConflictError",
    "ConflictErrorBody",
    "ContextWindowOverview",
    "CreateAgentRequestToolRulesItem",
    "CreateAssistantFileRequest",
    "CreateAssistantRequest",
    "CreateBlock",
    "DeleteAssistantFileResponse",
    "DeleteAssistantResponse",
    "E2BSandboxConfig",
    "EmbeddingConfig",
    "EmbeddingConfigEmbeddingEndpointType",
    "FileMetadata",
    "FunctionCallInput",
    "FunctionCallOutput",
    "FunctionSchema",
    "Health",
    "HttpValidationError",
    "InitToolRule",
    "InternalServerError",
    "InternalServerErrorBody",
    "Job",
    "JobStatus",
    "Letta",
    "LettaEnvironment",
    "LettaRequest",
    "LettaResponse",
    "LettaResponseMessagesItem",
    "LettaResponseMessagesItem_AssistantMessage",
    "LettaResponseMessagesItem_ReasoningMessage",
    "LettaResponseMessagesItem_SystemMessage",
    "LettaResponseMessagesItem_ToolCallMessage",
    "LettaResponseMessagesItem_ToolReturnMessage",
    "LettaResponseMessagesItem_UserMessage",
    "LettaSchemasLettaMessageToolCall",
    "LettaSchemasMessageMessage",
    "LettaSchemasOpenaiChatCompletionRequestTool",
    "LettaSchemasOpenaiChatCompletionRequestToolCall",
    "LettaSchemasOpenaiChatCompletionRequestToolCallFunction",
    "LettaSchemasOpenaiChatCompletionResponseMessage",
    "LettaSchemasOpenaiChatCompletionResponseToolCall",
    "LettaSchemasOpenaiChatCompletionsToolCallFunction",
    "LettaSchemasOpenaiChatCompletionsToolCallInput",
    "LettaSchemasOpenaiChatCompletionsToolCallOutput",
    "LettaSchemasToolTool",
    "LettaUsageStatistics",
    "LlmConfig",
    "LlmConfigModelEndpointType",
    "LocalSandboxConfig",
    "LogProbToken",
    "Memory",
    "MessageContentLogProb",
    "MessageCreate",
    "MessageCreateRole",
    "MessageRole",
    "NotFoundError",
    "NotFoundErrorBody",
    "NotFoundErrorBodyMessage",
    "OpenAiAssistant",
    "Organization",
    "OrganizationCreate",
    "Passage",
    "Provider",
    "ReasoningMessage",
    "RecallMemorySummary",
    "ResponseFormat",
    "SandboxConfig",
    "SandboxConfigCreate",
    "SandboxConfigCreateConfig",
    "SandboxConfigUpdate",
    "SandboxConfigUpdateConfig",
    "SandboxEnvironmentVariable",
    "SandboxEnvironmentVariableCreate",
    "SandboxEnvironmentVariableUpdate",
    "SandboxType",
    "Source",
    "SystemMessageInput",
    "SystemMessageOutput",
    "TerminalToolRule",
    "ToolCallDelta",
    "ToolCallFunctionOutput",
    "ToolCallMessage",
    "ToolCallMessageToolCall",
    "ToolCreate",
    "ToolFunctionChoice",
    "ToolInput",
    "ToolMessage",
    "ToolReturnMessage",
    "ToolReturnMessageStatus",
    "ToolRuleType",
    "ToolType",
    "UnprocessableEntityError",
    "UpdateAgentToolRulesItem",
    "UsageStatistics",
    "User",
    "UserCreate",
    "UserMessageInput",
    "UserMessageInputContent",
    "UserMessageOutput",
    "UserUpdate",
    "ValidationError",
    "ValidationErrorLocItem",
    "__version__",
    "agents",
    "blocks",
    "health",
    "jobs",
    "models",
    "providers",
    "sources",
    "tools",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from .system_message_input import SystemMessageInput
from .user_message_input import UserMessageInput
from .assistant_message_input import AssistantMessageInput
from .tool_message import ToolMessage

ChatCompletionRequestMessagesItem = typing.Union[
    SystemMessageInput, UserMessageInput, AssistantMessageInput, ToolMessage
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OpenAiAssistant(UncheckedBaseModel):
    """
    Represents an OpenAI assistant (equivalent to Letta preset)
    """

    id: str = pydantic.Field()
    """
    The unique identifier of the assistant.
    """

    name: str = pydantic.Field()
    """
    The name of the assistant.
    """

    object: typing.Optional[str] = None
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the assistant.
    """

    created_at: int = pydantic.Field()
    """
    The unix timestamp of when the assistant was created.
    """

    model: str = pydantic.Field()
    """
    The model used by the assistant.
    """

    instructions: str = pydantic.Field()
    """
    The instructions for the assistant.
    """

    tools: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The tools used by the assistant.
    """

    file_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of file IDs associated with the assistant.
    """

    metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Metadata associated with the assistant.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .chat_completion_request_messages_item import ChatCompletionRequestMessagesItem
from .response_format import ResponseFormat
from .chat_completion_request_stop import ChatCompletionRequestStop
from .tool_input import ToolInput
from .chat_completion_request_tool_choice import ChatCompletionRequestToolChoice
from .function_schema import FunctionSchema
from .chat_completion_request_function_call import ChatCompletionRequestFunctionCall
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ChatCompletionRequest(UncheckedBaseModel):
    """
    https://platform.openai.com/docs/api-reference/chat/create
    """

    model: str
    messages: typing.List[ChatCompletionRequestMessagesItem]
    frequency_penalty: typing.Optional[float] = None
    logit_bias: typing.Optional[typing.Dict[str, typing.Optional[int]]] = None
    logprobs: typing.Optional[bool] = None
    top_logprobs: typing.Optional[int] = None
    max_tokens: typing.Optional[int] = None
    n: typing.Optional[int] = None
    presence_penalty: typing.Optional[float] = None
    response_format: typing.Optional[ResponseFormat] = None
    seed: typing.Optional[int] = None
    stop: typing.Optional[ChatCompletionRequestStop] = None
    stream: typing.Optional[bool] = None
    temperature: typing.Optional[float] = None
    top_p: typing.Optional[float] = None
    user: typing.Optional[str] = None
    tools: typing.Optional[typing.List[ToolInput]] = None
    tool_choice: typing.Optional[ChatCompletionRequestToolChoice] = None
    functions: typing.Optional[typing.List[FunctionSchema]] = None
    function_call: typing.Optional[ChatCompletionRequestFunctionCall] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
import datetime as dt
from .message_role import MessageRole
from .letta_schemas_openai_chat_completions_tool_call_output import LettaSchemasOpenaiChatCompletionsToolCallOutput
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LettaSchemasMessageMessage(UncheckedBaseModel):
    """
    Letta's internal representation of a message. Includes methods to convert to/from LLM provider formats.

    Attributes:
        id (str): The unique identifier of the message.
        role (MessageRole): The role of the participant.
        text (str): The text of the message.
        user_id (str): The unique identifier of the user.
        agent_id (str): The unique identifier of the agent.
        model (str): The model used to make the function call.
        name (str): The name of the participant.
        created_at (datetime): The time the message was created.
        tool_calls (List[ToolCall]): The list of tool calls requested.
        tool_call_id (str): The id of the tool call.
    """

    created_by_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the user that made this object.
    """

    last_updated_by_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the user that made this object.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp when the object was created.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp when the object was last updated.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The human-friendly ID of the Message
    """

    role: MessageRole = pydantic.Field()
    """
    The role of the participant.
    """

    text: typing.Optional[str] = pydantic.Field(default=None)
    """
    The text of the message.
    """

    agent_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the agent.
    """

    model: typing.Optional[str] = pydantic.Field(default=None)
    """
    The model used to make the function call.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the participant.
    """

    tool_calls: typing.Optional[typing.List[LettaSchemasOpenaiChatCompletionsToolCallOutput]] = pydantic.Field(
        default=None
    )
    """
    The list of tool calls requested.
    """

    tool_call_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the tool call.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .agents_search_request_combinator import AgentsSearchRequestCombinator
from .agents_search_request_search_item import (
    AgentsSearchRequestSearchItem,
    AgentsSearchRequestSearchItem_Name,
    AgentsSearchRequestSearchItem_OrderBy,
    AgentsSearchRequestSearchItem_Version,
)
from .agents_search_request_search_item_name import AgentsSearchRequestSearchItemName
from .agents_search_request_search_item_name_operator import AgentsSearchRequestSearchItemNameOperator
from .agents_search_request_search_item_order_by import AgentsSearchRequestSearchItemOrderBy
from .agents_search_request_search_item_order_by_direction import AgentsSearchRequestSearchItemOrderByDirection
from .agents_search_request_search_item_order_by_value import AgentsSearchRequestSearchItemOrderByValue
from .agents_search_request_search_item_version import AgentsSearchRequestSearchItemVersion
from .create_agent_request_tool_rules_item import CreateAgentRequestToolRulesItem
from .update_agent_tool_rules_item import UpdateAgentToolRulesItem

__all__ = [
    "AgentsSearchRequestCombinator",
    "AgentsSearchRequestSearchItem",
    "AgentsSearchRequestSearchItemName",
    "AgentsSearchRequestSearchItemNameOperator",
    "AgentsSearchRequestSearchItemOrderBy",
    "AgentsSearchRequestSearchItemOrderByDirection",
    "AgentsSearchRequestSearchItemOrderByValue",
    "AgentsSearchRequestSearchItemVersion",
    "AgentsSearchRequestSearchItem_Name",
    "AgentsSearchRequestSearchItem_OrderBy",
    "AgentsSearchRequestSearchItem_Version",
    "CreateAgentRequestToolRulesItem",
    "UpdateAgentToolRulesItem",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from .tool_function_choice import ToolFunctionChoice

ChatCompletionRequestToolChoice = typing.Union[
    typing.Literal["none"], typing.Literal["auto"], typing.Literal["required"], ToolFunctionChoice
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .action_parameters_model import ActionParametersModel
from .action_response_model import ActionResponseModel
import typing_extensions
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ActionModel(UncheckedBaseModel):
    """
    Action data model.
    """

    name: str
    display_name: typing.Optional[str] = None
    parameters: ActionParametersModel
    response: ActionResponseModel
    app_name: typing_extensions.Annotated[str, FieldMetadata(alias="appName")]
    app_id: typing_extensions.Annotated[str, FieldMetadata(alias="appId")]
    tags: typing.List[str]
    enabled: typing.Optional[bool] = None
    logo: typing.Optional[str] = None
    description: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AgentsSearchRequestCombinator,
    AgentsSearchRequestSearchItem,
    AgentsSearchRequestSearchItemName,
    AgentsSearchRequestSearchItemNameOperator,
    AgentsSearchRequestSearchItemOrderBy,
    AgentsSearchRequestSearchItemOrderByDirection,
    AgentsSearchRequestSearchItemOrderByValue,
    AgentsSearchRequestSearchItemVersion,
    AgentsSearchRequestSearchItem_Name,
    AgentsSearchRequestSearchItem_OrderBy,
    AgentsSearchRequestSearchItem_Version,
    CreateAgentRequestToolRulesItem,
    UpdateAgentToolRulesItem,
)
from . import (
    archival_memory,
    context,
    core_memory,
    memory_variables,
    messages,
    recall_memory,
    sources,
    templates,
    tools,
)
from .memory_variables import MemoryVariablesGetResponse
from .messages import (
    LettaStreamingResponse,
    LettaStreamingResponse_AssistantMessage,
    LettaStreamingResponse_ReasoningMessage,
    LettaStreamingResponse_SystemMessage,
    LettaStreamingResponse_ToolCallMessage,
    LettaStreamingResponse_ToolReturnMessage,
    LettaStreamingResponse_UsageStatistics,
    LettaStreamingResponse_UserMessage,
    MessagesListResponse,
    MessagesListResponseItem,
    MessagesListResponseItem_AssistantMessage,
    MessagesListResponseItem_ReasoningMessage,
    MessagesListResponseItem_SystemMessage,
    MessagesListResponseItem_ToolCallMessage,
    MessagesListResponseItem_ToolReturnMessage,
    MessagesListResponseItem_UserMessage,
)
from .templates import TemplatesMigrateResponse

__all__ = [
    "AgentsSearchRequestCombinator",
    "AgentsSearchRequestSearchItem",
    "AgentsSearchRequestSearchItemName",
    "AgentsSearchRequestSearchItemNameOperator",
    "AgentsSearchRequestSearchItemOrderBy",
    "AgentsSearchRequestSearchItemOrderByDirection",
    "AgentsSearchRequestSearchItemOrderByValue",
    "AgentsSearchRequestSearchItemVersion",
    "AgentsSearchRequestSearchItem_Name",
    "AgentsSearchRequestSearchItem_OrderBy",
    "AgentsSearchRequestSearchItem_Version",
    "CreateAgentRequestToolRulesItem",
    "LettaStreamingResponse",
    "LettaStreamingResponse_AssistantMessage",
    "LettaStreamingResponse_ReasoningMessage",
    "LettaStreamingResponse_SystemMessage",
    "LettaStreamingResponse_ToolCallMessage",
    "LettaStreamingResponse_ToolReturnMessage",
    "LettaStreamingResponse_UsageStatistics",
    "LettaStreamingResponse_UserMessage",
    "MemoryVariablesGetResponse",
    "MessagesListResponse",
    "MessagesListResponseItem",
    "MessagesListResponseItem_AssistantMessage",
    "MessagesListResponseItem_ReasoningMessage",
    "MessagesListResponseItem_SystemMessage",
    "MessagesListResponseItem_ToolCallMessage",
    "MessagesListResponseItem_ToolReturnMessage",
    "MessagesListResponseItem_UserMessage",
    "TemplatesMigrateResponse",
    "UpdateAgentToolRulesItem",
    "archival_memory",
    "context",
    "core_memory",
    "memory_variables",
    "messages",
    "recall_memory",
    "sources",
    "templates",
    "tools",
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .letta_response_messages_item import LettaResponseMessagesItem
import pydantic
from .letta_usage_statistics import LettaUsageStatistics
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LettaResponse(UncheckedBaseModel):
    """
    Response object from an agent interaction, consisting of the new messages generated by the agent and usage statistics.
    The type of the returned messages can be either `Message` or `LettaMessage`, depending on what was specified in the request.

    Attributes:
        messages (List[Union[Message, LettaMessage]]): The messages returned by the agent.
        usage (LettaUsageStatistics): The usage statistics
    """

    messages: typing.List[LettaResponseMessagesItem] = pydantic.Field()
    """
    The messages returned by the agent.
    """

    usage: LettaUsageStatistics = pydantic.Field()
    """
    The usage statistics of the agent.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AppAuthSchemeAuthMode = typing.Union[
    typing.Literal["OAUTH2", "OAUTH1", "API_KEY", "BASIC", "BEARER_TOKEN", "BASIC_WITH_JWT"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .letta_schemas_openai_chat_completion_response_message import LettaSchemasOpenaiChatCompletionResponseMessage
import typing
from .message_content_log_prob import MessageContentLogProb
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class Choice(UncheckedBaseModel):
    finish_reason: str
    index: int
    message: LettaSchemasOpenaiChatCompletionResponseMessage
    logprobs: typing.Optional[typing.Dict[str, typing.Optional[typing.List[MessageContentLogProb]]]] = None
    seed: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from ..types.job import Job
from ..core.unchecked_base_model import construct_type
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.jsonable_encoder import jsonable_encoder
from ..core.client_wrapper import AsyncClientWrapper


class JobsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, source_id: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Job]:
        """
        List all jobs.

        Parameters
        ----------
        source_id : typing.Optional[str]
            Only list jobs associated with the source.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Job]
            Successful Response

        Examples
        --------
        from letta import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.jobs.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/jobs/",
            method="GET",
            params={
                "source_id": source_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Job],
                    construct_type(
                        type_=typing.List[Job],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_active(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[Job]:
        """
        List all active jobs.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Job]
            Successful Response

        Examples
        --------
        from letta import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.jobs.list_active()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/jobs/active",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Job],
                    construct_type(
                        type_=typing.List[Job],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Job:
        """
        Get the status of a job.

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Job
            Successful Response

        Examples
        --------
        from letta import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.jobs.get(
            job_id="job_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/jobs/{jsonable_encoder(job_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Job,
                    construct_type(
                        type_=Job,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Job:
        """
        Delete a job by its job_id.

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Job
            Successful Response

        Examples
        --------
        from letta import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.jobs.delete(
            job_id="job_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/jobs/{jsonable_encoder(job_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Job,
                    construct_type(
                        type_=Job,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncJobsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, source_id: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Job]:
        """
        List all jobs.

        Parameters
        ----------
        source_id : typing.Optional[str]
            Only list jobs associated with the source.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Job]
            Successful Response

        Examples
        --------
        import asyncio

        from letta import AsyncLetta

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.jobs.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/jobs/",
            method="GET",
            params={
                "source_id": source_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Job],
                    construct_type(
                        type_=typing.List[Job],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_active(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[Job]:
        """
        List all active jobs.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Job]
            Successful Response

        Examples
        --------
        import asyncio

        from letta import AsyncLetta

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.jobs.list_active()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/jobs/active",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Job],
                    construct_type(
                        type_=typing.List[Job],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Job:
        """
        Get the status of a job.

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Job
            Successful Response

        Examples
        --------
        import asyncio

        from letta import AsyncLetta

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.jobs.get(
                job_id="job_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/jobs/{jsonable_encoder(job_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Job,
                    construct_type(
                        type_=Job,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Job:
        """
        Delete a job by its job_id.

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Job
            Successful Response

        Examples
        --------
        import asyncio

        from letta import AsyncLetta

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.jobs.delete(
                job_id="job_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/jobs/{jsonable_encoder(job_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Job,
                    construct_type(
                        type_=Job,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .choice import Choice
import datetime as dt
from .usage_statistics import UsageStatistics
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ChatCompletionResponse(UncheckedBaseModel):
    """
    https://platform.openai.com/docs/api-reference/chat/object
    """

    id: str
    choices: typing.List[Choice]
    created: dt.datetime
    model: typing.Optional[str] = None
    system_fingerprint: typing.Optional[str] = None
    object: typing.Optional[typing.Literal["chat.completion"]] = None
    usage: UsageStatistics

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from .agents_search_request_search_item_name_operator import AgentsSearchRequestSearchItemNameOperator
from .agents_search_request_search_item_order_by_value import AgentsSearchRequestSearchItemOrderByValue
from .agents_search_request_search_item_order_by_direction import AgentsSearchRequestSearchItemOrderByDirection
import typing_extensions
from ...core.unchecked_base_model import UnionMetadata


class AgentsSearchRequestSearchItem_Version(UncheckedBaseModel):
    field: typing.Literal["version"] = "version"
    value: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AgentsSearchRequestSearchItem_Name(UncheckedBaseModel):
    field: typing.Literal["name"] = "name"
    operator: AgentsSearchRequestSearchItemNameOperator
    value: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AgentsSearchRequestSearchItem_OrderBy(UncheckedBaseModel):
    field: typing.Literal["order_by"] = "order_by"
    value: AgentsSearchRequestSearchItemOrderByValue
    direction: AgentsSearchRequestSearchItemOrderByDirection

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


AgentsSearchRequestSearchItem = typing_extensions.Annotated[
    typing.Union[
        AgentsSearchRequestSearchItem_Version, AgentsSearchRequestSearchItem_Name, AgentsSearchRequestSearchItem_OrderBy
    ],
    UnionMetadata(discriminant="field"),
]
